==> live-wallpaper_0.7.tar.gz: live-wallpaper-0.7/src/livewp-main.c <==
/* vim: set sw=4 ts=4 et: */
/*
 * This file is part of Live Wallpaper (livewp)
 * 
 * Copyright (C) 2010 Vlad Vasiliev
 * Copyright (C) 2010 Tanya Makova
 *       for the code
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
*/
/*******************************************************************************/
#include "livewp-common.h"
#include "livewp-scene.h"
#include "livewp-settings.h"
#include "livewp-home-widget.h"
/*******************************************************************************/

static void set_live_bg (Display *display, Window xwindow, int mode);
static 
void set_live_bg (Display *display, Window xwindow, int mode)
{
        Atom atom;

        atom = XInternAtom (display, "_HILDON_LIVE_DESKTOP_BACKGROUND", False);
        fprintf (stderr, "XID: 0x%x\n", (unsigned)xwindow);

        XChangeProperty (display,
                       xwindow,
                       atom,
                       XA_INTEGER, 32, PropModeReplace,
                       (unsigned char *) &mode, 1);
}
/*******************************************************************************/

void
view_state_changed (Animation_WallpaperPrivate *priv)
{
   fprintf(stderr," VIew %i State %i\n", priv->view, priv->visible);
   if (priv->visible){
       if (!priv->running){
            if (!strcmp(priv->theme, "Xsnow")){
                init_scene_Xsnow(priv->desktop_plugin);
            }
       }
       if (!strcmp(priv->theme,"Video")){
           if (priv->pipeline){
               gst_element_set_state (priv->pipeline, GST_STATE_PLAYING);
           }else{
               init_scene_Video(priv->desktop_plugin);
           }
       }
       if (priv->long_timer == 0 ){
            priv->long_timer = g_timeout_add(LONG_TIMER, (GtkFunction)long_timeout, priv->desktop_plugin);
            run_long_timeout(priv->desktop_plugin);
       }
   }else{
        if (priv->pipeline){
           gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
           return;
        }
        if (priv->long_timer != 0 ){
            g_source_remove(priv->long_timer);
            priv->long_timer = 0;
        }
   }
}
/*******************************************************************************/
gboolean
check_alive_event(Animation_WallpaperPrivate *priv) 
{
    if ( time(NULL) - priv->last_alive_event > 10*60) /* If last event later 10 minute */
        quit_from_program(priv);
    return TRUE;
}

/*******************************************************************************/
void
view_changed (GConfClient *client, guint cnxn_id,
                 GConfEntry *entry, Animation_WallpaperPrivate *priv)
{
   if (entry->value->type == GCONF_VALUE_INT) {
        if (priv->view == gconf_value_get_int(entry->value)){
            if (priv->visible != TRUE){
                priv->visible = TRUE;
                view_state_changed(priv);
            }
        }else{
            if (priv->visible != FALSE){
                priv->visible = FALSE;
                view_state_changed(priv);
            }
        }
    }
}
/*******************************************************************************/
void
quit_from_program (Animation_WallpaperPrivate *priv)
{
     fprintf(stderr,"quit_from_program\n");
     if (priv){
	     livewp_deinitialize_dbus(priv);
         if (priv->view_notify != 0) { 
            gconf_client_notify_remove (gconf_client_get_default (), priv->view_notify);
            priv->view_notify = 0;
         }
	     if (priv->long_timer){
		    g_source_remove(priv->long_timer);
	    	priv->long_timer = 0;
	     }
	     if (priv->short_timer){
		    g_source_remove(priv->short_timer);
		    priv->short_timer = 0;
	     }
	     destroy_scene(priv->desktop_plugin);
         osso_deinitialize(priv->osso);
         g_hash_table_destroy(priv->hash_scene_func);
         g_free(priv->desktop_plugin);
         priv->desktop_plugin = NULL;
         g_free(priv);
         priv = NULL;
     } 
     fprintf(stderr,"end of quit_from_program\n");
     gtk_main_quit();;
}
/*******************************************************************************/
int
main(int argc, char *argv[])
{
    GtkWidget *main_widget = NULL;
    HildonProgram       *app;
    int view = 1;

     
    if (argc == 2) 
        view = atoi(argv[1]);
    if (view < 1 || view > 4)
        view = 1;

    AWallpaperPlugin *desktop_plugin = g_new0 (AWallpaperPlugin, 1);
    Animation_WallpaperPrivate *priv = g_new0 (Animation_WallpaperPrivate, 1);
    desktop_plugin->priv = priv; 

    gst_init (NULL, NULL);

    /* Ininitializing */
    hildon_gtk_init (&argc, &argv);
    app = HILDON_PROGRAM (hildon_program_get_instance());
    g_set_application_name (PACKAGE);
 
    priv->osso = osso_initialize("org.maemo.livewp", VERSION, TRUE, NULL);
    if(!priv->osso){
        fprintf(stderr,"osso_initialize failed\n");
        return 1;
    }

#ifdef ENABLE_NLS
    setlocale(LC_ALL, "");
    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);
#endif
/* Create Main GUI */
    main_widget = hildon_window_new ();
    gtk_window_set_title(GTK_WINDOW(main_widget), PACKAGE);
    gtk_window_fullscreen(GTK_WINDOW(main_widget));
    
    gtk_widget_show_all(GTK_WIDGET(main_widget));

    priv->hash_scene_func = g_hash_table_new(g_str_hash, g_str_equal);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Accel"),  (gpointer)&init_scene_Accel);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Berlin"), (gpointer)&init_scene_Berlin);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Modern"), (gpointer)&init_scene_Modern);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Matrix"), (gpointer)&init_scene_Matrix);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Video"),  (gpointer)&init_scene_Video);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Xsnow"),  (gpointer)&init_scene_Xsnow);

    priv->scene = NULL;
    priv->pipeline = NULL;
    priv->podpid = -1;
    priv->running = FALSE;
    priv->window = main_widget;
    /* fprintf(stderr,"XWINDOW %i\n",GDK_WINDOW_XID (main_widget->window)); */
    priv->desktop_plugin = desktop_plugin;
    priv->view = view;

    if (current_active_view() == view) 
        priv->visible = TRUE;
    else 
        priv->visible = FALSE;

    /* Load config */
    read_config(priv);

    /* Initialize DBUS */
    livewp_initialize_dbus(priv);
    set_live_bg(GDK_WINDOW_XDISPLAY (main_widget->window), GDK_WINDOW_XID (main_widget->window), view);
    
    priv->view_notify = 0; 
    /* fprintf(stderr,"VISIBLE %i %i\n",priv->visible, current_active_view()); */
    gconf_client_add_dir(gconf_client_get_default (), "/apps/osso/hildon-desktop/views", GCONF_CLIENT_PRELOAD_NONE, NULL);
    priv->view_notify = gconf_client_notify_add(gconf_client_get_default (),"/apps/osso/hildon-desktop/views/current",
                                                (GConfClientNotifyFunc)view_changed, priv, NULL, NULL);

    /* Set start position of Video to zero */
    if (!strcmp(priv->theme,"Video"))
        priv->theme_int_parametr1 = 0;

    init_scene_theme(desktop_plugin);
        
    priv->long_timer = g_timeout_add(LONG_TIMER, (GtkFunction)long_timeout, desktop_plugin);

    priv->alive_timer = g_timeout_add(60000*10, (GtkFunction)check_alive_event, priv);  /* One per 10 minute */ 
    gtk_main();
    return 0;
}

==> live-wallpaper_0.7.1.tar.gz: live-wallpaper-0.7.1/src/livewp-main.c <==
/* vim: set sw=4 ts=4 et: */
/*
 * This file is part of Live Wallpaper (livewp)
 * 
 * Copyright (C) 2010 Vlad Vasiliev
 * Copyright (C) 2010 Tanya Makova
 *       for the code
 * 
 * This software is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
*/
/*******************************************************************************/
#include "livewp-common.h"
#include "livewp-scene.h"
#include "livewp-settings.h"
#include "livewp-home-widget.h"
/*******************************************************************************/

static void set_live_bg (Display *display, Window xwindow, int mode);
void quit_from_program (Animation_WallpaperPrivate *priv);
void view_state_changed (Animation_WallpaperPrivate *priv);

static 
void set_live_bg (Display *display, Window xwindow, int mode)
{
        Atom atom;

        atom = XInternAtom (display, "_HILDON_LIVE_DESKTOP_BACKGROUND", False);
        fprintf (stderr, "XID: 0x%x\n", (unsigned)xwindow);

        XChangeProperty (display,
                       xwindow,
                       atom,
                       XA_INTEGER, 32, PropModeReplace,
                       (unsigned char *) &mode, 1);
}
/*******************************************************************************/

void
view_state_changed (Animation_WallpaperPrivate *priv)
{
   fprintf(stderr," VIew %i State %i\n", priv->view, priv->visible);
   if (priv->visible){
       if (!priv->running){
            if (!strcmp(priv->theme, "Xsnow")){
                init_scene_Xsnow(priv->desktop_plugin);
            }
       }
       if (!strcmp(priv->theme,"Video")){
           if (priv->pipeline){
               gst_element_set_state (priv->pipeline, GST_STATE_PLAYING);
           }else{
               init_scene_Video(priv->desktop_plugin);
           }
       }
       if (priv->long_timer == 0 ){
            priv->long_timer = g_timeout_add(LONG_TIMER, (GtkFunction)long_timeout, priv->desktop_plugin);
            run_long_timeout(priv->desktop_plugin);
       }else {
            run_long_timeout(priv->desktop_plugin);
       }
   }else{
        if (priv->pipeline){
           gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
           return;
        }
        if (priv->long_timer != 0 ){
            g_source_remove(priv->long_timer);
            priv->long_timer = 0;
        }
   }
}
/*******************************************************************************/
gboolean
check_alive_event(Animation_WallpaperPrivate *priv) 
{
    if ( time(NULL) - priv->last_alive_event > 10*60) /* If last event later 10 minute */
        quit_from_program(priv);
    return TRUE;
}

/*******************************************************************************/
void
view_changed (GConfClient *client, guint cnxn_id,
                 GConfEntry *entry, Animation_WallpaperPrivate *priv)
{
   if (entry->value->type == GCONF_VALUE_INT) {
        if (priv->view == gconf_value_get_int(entry->value)){
            if (priv->visible != TRUE){
                priv->visible = TRUE;
                view_state_changed(priv);
            }
        }else{
            if (priv->visible != FALSE){
                priv->visible = FALSE;
                view_state_changed(priv);
            }
        }
    }
}
/*******************************************************************************/
void
quit_from_program (Animation_WallpaperPrivate *priv)
{
     fprintf(stderr,"quit_from_program\n");
     if (priv){
	     livewp_deinitialize_dbus(priv);
         if (priv->view_notify != 0) { 
            gconf_client_notify_remove (gconf_client_get_default (), priv->view_notify);
            priv->view_notify = 0;
         }
	     if (priv->long_timer){
		    g_source_remove(priv->long_timer);
	    	priv->long_timer = 0;
	     }
	     if (priv->short_timer){
		    g_source_remove(priv->short_timer);
		    priv->short_timer = 0;
	     }
	     destroy_scene(priv->desktop_plugin);
         osso_deinitialize(priv->osso);
         g_hash_table_destroy(priv->hash_scene_func);
         g_free(priv->desktop_plugin);
         priv->desktop_plugin = NULL;
         g_free(priv);
         priv = NULL;
     } 
     fprintf(stderr,"end of quit_from_program\n");
     gtk_main_quit();;
}
/*******************************************************************************/
int
main(int argc, char *argv[])
{
    GtkWidget *main_widget = NULL;
    HildonProgram       *app;
    int view = 1;

     
    if (argc == 2) 
        view = atoi(argv[1]);
    if (view < 1 || view > 4)
        view = 1;

    AWallpaperPlugin *desktop_plugin = g_new0 (AWallpaperPlugin, 1);
    Animation_WallpaperPrivate *priv = g_new0 (Animation_WallpaperPrivate, 1);
    desktop_plugin->priv = priv; 

    gst_init (NULL, NULL);

    /* Ininitializing */
    hildon_gtk_init (&argc, &argv);
    app = HILDON_PROGRAM (hildon_program_get_instance());
    g_set_application_name (PACKAGE);
 
    priv->osso = osso_initialize("org.maemo.livewp", VERSION, TRUE, NULL);
    if(!priv->osso){
        fprintf(stderr,"osso_initialize failed\n");
        return 1;
    }

#ifdef ENABLE_NLS
    setlocale(LC_ALL, "");
    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);
#endif
/* Create Main GUI */
    main_widget = hildon_window_new ();
    gtk_window_set_title(GTK_WINDOW(main_widget), PACKAGE);
    gtk_window_fullscreen(GTK_WINDOW(main_widget));
    
    gtk_widget_show_all(GTK_WIDGET(main_widget));

    priv->hash_scene_func = g_hash_table_new(g_str_hash, g_str_equal);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Accel"),  (gpointer)&init_scene_Accel);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Berlin"), (gpointer)&init_scene_Berlin);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Modern"), (gpointer)&init_scene_Modern);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Matrix"), (gpointer)&init_scene_Matrix);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Video"),  (gpointer)&init_scene_Video);
    g_hash_table_insert(priv->hash_scene_func, g_strdup("Xsnow"),  (gpointer)&init_scene_Xsnow);

    priv->scene = NULL;
    priv->pipeline = NULL;
    priv->podpid = -1;
    priv->running = FALSE;
    priv->window = main_widget;
    /* fprintf(stderr,"XWINDOW %i\n",GDK_WINDOW_XID (main_widget->window)); */
    priv->desktop_plugin = desktop_plugin;
    priv->view = view;

    if (current_active_view() == view) 
        priv->visible = TRUE;
    else 
        priv->visible = FALSE;

    /* Load config */
    read_config(priv);

    /* Initialize DBUS */
    livewp_initialize_dbus(priv);
    set_live_bg(GDK_WINDOW_XDISPLAY (main_widget->window), GDK_WINDOW_XID (main_widget->window), view);
    
    priv->view_notify = 0; 
    /* fprintf(stderr,"VISIBLE %i %i\n",priv->visible, current_active_view()); */
    gconf_client_add_dir(gconf_client_get_default (), "/apps/osso/hildon-desktop/views", GCONF_CLIENT_PRELOAD_NONE, NULL);
    priv->view_notify = gconf_client_notify_add(gconf_client_get_default (),"/apps/osso/hildon-desktop/views/current",
                                                (GConfClientNotifyFunc)view_changed, priv, NULL, NULL);

    /* Set start position of Video to zero */
    if (!strcmp(priv->theme,"Video"))
        priv->theme_int_parametr1 = 0;

    init_scene_theme(desktop_plugin);
        
    priv->long_timer = g_timeout_add(LONG_TIMER, (GtkFunction)long_timeout, desktop_plugin);

    priv->alive_timer = g_timeout_add(60000*10, (GtkFunction)check_alive_event, priv);  /* One per 10 minute */ 
    gtk_main();
    return 0;
}
